package com.winhc.service.impl;

import com.winhc.common.enums.CompanyEnum;
import com.winhc.service.RelationService;
import com.winhc.utils.CompanyUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.neo4j.driver.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author π
 * @Description:股东关系 人->公司
 * @date 2021/1/11 10:03
 */
@Slf4j
@Service("holderRelationV1ServiceImpl")
@AllArgsConstructor
public class HolderRelationV1ServiceImpl implements RelationService {

    @Autowired
    @Qualifier("DriverV1")
    Driver driver;

    @Override
    public String save(List<Map<String, Object>> batch_list) {
        if (batch_list.isEmpty()) return null;
        long start = System.currentTimeMillis();
        Session session = driver.session();
        final String cql = "WITH  {batch_list} AS batch_list \n" +
                "UNWIND batch_list AS row \n" +
                "MERGE(s:" + CompanyEnum.Lable.PERSON.code + "{person_id:row.start_id}) \n" +
                "SET s.name=row.start_name, s.person_id=row.start_id \n" +
                "SET s:" + CompanyUtils.getIncrPersonLabelV2("新增", CompanyEnum.SPLIT_HOUR) + " \n" +
                "MERGE(e:" + CompanyEnum.Lable.COMPANY.code + "{company_id:row.end_id}) \n" +
                "SET e.name=row.end_name, e.company_id=row.end_id \n" +
                "WITH s,e,row \n" +
                "MERGE(s)-[r:" + CompanyEnum.Lable.投资.code + "]->(e) \n" +
                "SET r.percent=row.percent, r.deleted=row.deleted \n";
        log.info("consumer size: {}, cql:{}", batch_list.size(), cql);
        String data = CompanyUtils.writeNeo4j(session, cql, new HashMap<String, Object>() {{
            put("batch_list", batch_list);
        }});
        session.close();
        log.info("class:{} | save size:{} | cost:{}", HolderRelationV1ServiceImpl.class.getSimpleName(), batch_list.size(), (System.currentTimeMillis() - start));
        return data;
    }
}
