package com.winhc.service.impl;

import com.alibaba.fastjson.JSON;
import com.winhc.common.enums.CompanyEnum;
import com.winhc.service.RelationService;
import com.winhc.utils.CompanyUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.neo4j.driver.Driver;
import org.neo4j.driver.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author π
 * @Description:股东关系 人->公司
 * @date 2021/1/11 10:03
 */
@Slf4j
@Service("personCompanysImpl")
@AllArgsConstructor
public class PersonCompanysImpl implements RelationService {

    @Autowired
    @Qualifier("DriverV1")
    Driver driver;

    @Override
    public String save(List<Map<String, Object>> batch_list) {
        if (batch_list.isEmpty()) return null;
        long start = System.currentTimeMillis();
        Session session = driver.session();
//        final String cql =
//                        "\nWITH  {batch_list} AS batch_list \n" +
//                           "UNWIND batch_list AS row \n" +
//                           "MATCH (n:企业)-[r]-(m:个人) \n" +
//                           "WHERE n.company_id in split(row.company_ids,'&')  and m.name in [row.name] \n" +
//                           "return m.name as name\n";
//                           //"return m.name as name,collect(distinct m) as nodes\n";
        System.out.println(JSON.toJSONString(batch_list));
        final String cql =
                "\nWITH  {batch_list} AS batch_list \n" +
                        "UNWIND batch_list AS row \n" +
                        "MATCH (n:企业)<-[r]-(m:个人{name:row.name}) \n" +
                        "WHERE n.company_id in split(row.company_ids,'&')\n" +
                        "WITH m.name as name,collect(distinct m) as nodes\n" +
                        "WITH head(nodes) as first_node, nodes\n" +
                        "UNWIND nodes as other_node\n" +
                        "MATCH (other_node)-[r]-(x:企业)\n" +
                        "WHERE other_node.person_id <> first_node.person_id\n" +
                        "WITH first_node,r,other_node,x\n" +
                        "CALL apoc.merge.relationship(first_node, TYPE(r), properties(r),{}, x,{}) YIELD rel\n" +
                        "SET first_node:合并20210409\n" +
                        "SET other_node:删除20210409\n" +
                        "DELETE r";
        log.info("consumer size: {}, cql:{}", batch_list.size(), cql);
        String data = CompanyUtils.writeNeo4j(session, cql, new HashMap<String, Object>() {{
            put("batch_list", batch_list);
        }});
        session.close();
        log.info("class:{} | save size:{} | cost:{}", PersonCompanysImpl.class.getSimpleName(), batch_list.size(), (System.currentTimeMillis() - start));
        return data;
    }
}
