package com.winhc.service.impl;

import com.winhc.common.enums.CompanyEnum;
import com.winhc.config.ConfigConstant;
import com.winhc.kafka.KafkaProduce;
import com.winhc.service.RelationService;
import com.winhc.utils.CompanyUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.neo4j.driver.Driver;
import org.neo4j.driver.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author π
 * @Description:全量人员合并处理逻辑
 * @date 2022/5/12 10:03
 */
@Slf4j
@Service("personMergeAllV2Impl")
@AllArgsConstructor
public class PersonMergeAllV2Impl implements RelationService {

    @Autowired
    @Qualifier("DriverV1")
    Driver driver;
    @Autowired
    KafkaProduce kafkaProduce;
    @Autowired
    ConfigConstant configConstant;

    @Override
    public String save(List<Map<String, Object>> batch_list) {
        if (batch_list.isEmpty()) return null;
        long start = System.currentTimeMillis();
        Session session = driver.session();
        //1.查询关联人员
        final String merge_cql =
                "\nWITH  {merge_list} AS batch_list \n" +
                        "UNWIND batch_list AS row\n" +
                        "MATCH (c0:" + CompanyEnum.Lable.COMPANY.code + "{company_id:row.first_company_id})-[r0]-(p:" + CompanyEnum.Lable.PERSON.code + "{name:row.first_human_name})\n" +
                        ", (q:" + CompanyEnum.Lable.PERSON.code + " {person_id:row.second_human_pid})-[r]-(x:" + CompanyEnum.Lable.COMPANY.code + ")\n" +
                        "WHERE p.person_id <> q.person_id\n" +
                        "WITH p as first_node, q as other_node, x, r\n" +
                        "CALL apoc.merge.relationship.eager(first_node, TYPE(r), properties(r),{}, x,{}) YIELD rel\n" +
                        "SET first_node:" + CompanyUtils.getIncrPersonLabelV2("新增", CompanyEnum.SPLIT_HOUR) + "\n" +
                        "SET other_node:" + CompanyUtils.getIncrPersonLabelV2("删除", CompanyEnum.SPLIT_HOUR) + "\n" +
                        "DELETE r\n" +
                        "RETURN first_node.person_id as new_human_pid,first_node.name as new_human_name,other_node.person_id as old_human_pid,other_node.name as old_human_name";
        log.info("merge size: {}, cql2:{}", batch_list.size(), merge_cql);
        String data = CompanyUtils.writeNeo4j2(session, merge_cql, new HashMap<String, Object>() {{
            put("merge_list", batch_list);
        }});

        session.close();
        log.info("class:{} | save size:{} |  merge size:{} |cost:{}", PersonMergeAllV2Impl.class.getSimpleName(), batch_list.size(), batch_list.size(), (System.currentTimeMillis() - start));

        //2.发送变更人员记录
        if (StringUtils.isNotBlank(data)) {
            kafkaProduce.produce(configConstant.topic_pid_update_v1, data);
        }
        return data;
    }


}
