package com.winhc.utils;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.ImmutableMap;
import com.winhc.common.enums.CompanyEnum;
import com.winhc.db.mongodb.dataobject.NodeRelationError;
import com.winhc.pojo.MergeAllPerson;
import com.winhc.pojo.MergePerson;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.neo4j.driver.Record;
import org.neo4j.driver.Result;
import org.neo4j.driver.Session;

import java.util.*;
import java.util.stream.Collectors;

import static com.winhc.utils.DateUtil.formatDate_YYYY_MM_DD_HH_MM_SS;
import static java.util.stream.Collectors.toList;

/**
 * @author π
 * @Description:
 * @date 2021/1/11 21:31
 */
@Slf4j
public class CompanyUtils {
    public static List<Map<String, Object>> map(List<ConsumerRecord<?, ?>> records) {
        return records.stream().filter(r -> (r != null && r.value() != null)).map(r -> {
            Map<String, Object> m = JSONUtil.parseObj(r.value().toString());
            return m;
        }).collect(toList());
    }

    public static List<Map<String, Object>> mapPlus(List<ConsumerRecord<?, ?>> records) {
        return records.stream().filter(r -> (r != null && r.value() != null)).map(r -> {
            JSONObject jsonObject = JSONObject.parseObject(r.value().toString());
            String person_name = jsonObject.getString("person_name");
            JSONArray company_infos = jsonObject.getJSONArray("company_infos");
            ArrayList<String> list = new ArrayList<>();
            HashMap<String, Object> map = new HashMap<>();
            for (Object company_info : company_infos) {
                JSONObject jsonObject1 = (JSONObject) company_info;
                String company_id = jsonObject1.getString("company_id");
                list.add(company_id);
            }
            map.put("name", person_name);
            map.put("company_ids", listToString(list));
            return map;
        }).collect(toList());
    }

    public static List<Map<String, Object>> mapPlusV1(List<ConsumerRecord<?, ?>> records) {
        return records.stream().filter(r -> (r != null && r.value() != null)).map(r -> {
            HashMap<String, Object> map = new HashMap<>();
            JSONObject jsonObject = JSONObject.parseObject(r.value().toString());
            String merge_human_pid = jsonObject.getString("merge_human_pid");
            String deleted_human_pid = jsonObject.getString("deleted_human_pid");

            map.put("merge_human_pid", merge_human_pid);
            map.put("deleted_human_pid", deleted_human_pid);
            return map;
        }).collect(toList());
    }

    public static String listToString(List<String> list) {
        if (list.size() == 0) return "";
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < list.size(); i++) {
            sb.append(list.get(i));
            if (i != list.size() - 1) {
                sb.append("&");
            }
        }
        return sb.toString();
    }

    public static List<Map<String, Object>> filterList(List<Map<String, Object>> list, String type) {
        return list.stream().filter(r -> (r.getOrDefault("topic_type", "-1").equals(type))).collect(toList());
    }

    public static List<Map<String, Object>> getMergeIds2(List<Map<String, Object>> list) {
        return list.stream()
                .filter(r -> r.getOrDefault("start_id", "0").toString().length() == 33)
                .collect(Collectors.toMap(t -> t.getOrDefault("start_id", "0").toString(), t -> t, (n, o) -> n))
                .values().stream().map(x -> {
                    Map<String, Object> m = x;
                    String person_id = m.get("start_id").toString();
                    String person_name = m.get("start_name").toString();
                    return ImmutableMap.<String, Object>of(
                            "_id", person_id,
                            "update_time", formatDate_YYYY_MM_DD_HH_MM_SS(),
                            "data", ImmutableMap.of(
                                    "person_id", person_id
                                    , "name", person_name
                                    , "topic_type", "800"
                            )
                    );
                }).collect(toList());
    }

    public static List<NodeRelationError> toMessage(List<ConsumerRecord> records, String errorMessage) {
        return records.stream().filter(r -> (r != null && r.value() != null)).map(r -> {
            String consumerMessage = r.value().toString();
            Map<String, Object> m = JSONUtil.parseObj(consumerMessage);
            Object topic_type = m.getOrDefault("topic_type", "-1");
            Date date = new Date();
            NodeRelationError n1 = new NodeRelationError();
            n1.setCreateTime(date);
            n1.setUpdateTime(date);
            n1.setConsumerMessage(consumerMessage);
            n1.setErrorMessage(errorMessage);
            n1.setStatus(0);
            n1.setTopicType(String.valueOf(topic_type));
            return n1;
        }).collect(toList());
    }

    public static List<String> toMessage(List<ConsumerRecord> records) {
        return records.stream()
                .filter(r -> (r != null && r.value() != null))
                .map(r -> r.value().toString())
                .collect(toList());
    }

    public static String writeNeo4j(Session session, String cql, HashMap<String, Object> parameters) {
        String data = session.writeTransaction(tx -> {
            Result result = tx.run(cql, parameters);
            return "success";
        });
        return data;
    }

    public static String writeNeo4j2(Session session, String cql, HashMap<String, Object> parameters) {
        List<Record> dataList = getRecords(session, cql, parameters);
        Map<String, MergePerson> data = dataList.stream()
                .map(record -> JSON.parseObject(JSON.toJSONString(record.asMap()), MergePerson.class))
                .collect(Collectors.toMap(MergePerson::getId, t -> t, (n, o) -> n));
        Collection<MergePerson> values = data.values();
        return values.size() > 0 ? JSONObject.toJSONString(data.values()) : null;
    }

    public static List<Map<String, String>> writeNeo4j1(Session session, String cql, HashMap<String, Object> parameters) {
        Collection<List<MergePerson>> collect = getMergePerson(session, cql, parameters);
        return trans(collect);

    }

    public static List<Map<String, String>> trans(Collection<List<MergePerson>> collect) {
        List<Map<String, String>> list = new ArrayList<>();
        collect.forEach(v -> v.stream()
                .max(Comparator.comparing(MergePerson::getCnt))
                .ifPresent(maxPerson -> v.forEach(m -> {
                    if (!m.getOld_human_pid().equals(maxPerson.getOld_human_pid())) {
                        HashMap<String, String> map = new HashMap<>();
                        map.put("new_human_pid", maxPerson.getOld_human_pid());
                        map.put("new_human_name", maxPerson.getOld_human_name());
                        map.put("old_human_pid", m.getOld_human_pid());
                        map.put("old_human_name", m.getOld_human_name());
                        list.add(map);
                    }
                })));
        return list.stream().collect(
                Collectors.collectingAndThen(
                        Collectors.toCollection(
                                () -> new TreeSet<>(Comparator.comparing(m -> m.get("new_human_pid") + m.get("old_human_pid")))
                        ), ArrayList::new
                )
        );
    }

    public static Collection<List<MergePerson>> getMergePerson(Session session, String cql, HashMap<String, Object> parameters) {
        List<Record> dataList = getRecords(session, cql, parameters);
        return dataList.stream()
                .map(record -> JSON.parseObject(JSON.toJSONString(record.asMap()), MergePerson.class))
                .collect(Collectors.groupingBy(MergePerson::getNew_human_pid)).values();
    }

    private static List<Record> getRecords(Session session, String cql, HashMap<String, Object> parameters) {
        return session.writeTransaction(tx -> {
            Result result = tx.run(cql, parameters);
            return result.list();
        });
    }

    public static List<Map<String, String>> writeNeo4j3(Session session, String cql, HashMap<String, Object> parameters) {
        List<Record> dataList = getRecords(session, cql, parameters);
        Set<String> newIds = new HashSet<>();
        Set<String> oldIds = new HashSet<>();
        List<Map<String, String>> list = dataList.stream()
                .map(record -> JSON.parseObject(JSON.toJSONString(record.asMap()), MergeAllPerson.class))
                .filter(m -> !m.getNew_human_pid().equals(m.getOld_human_pid()))
                .map(x -> {
                    HashMap<String, String> map = new HashMap<>();
                    if (x.getNew_cnt() >= x.getOld_cnt()) {
                        map.put("new_human_pid", x.getNew_human_pid());
                        map.put("old_human_pid", x.getOld_human_pid());
                        newIds.add(x.getNew_human_pid());
                        oldIds.add(x.getOld_human_pid());
                    } else {
                        map.put("new_human_pid", x.getOld_human_pid());
                        map.put("old_human_pid", x.getNew_human_pid());
                        newIds.add(x.getOld_human_pid());
                        oldIds.add(x.getNew_human_pid());
                    }
                    return map;
                })
                .collect(
                        Collectors.collectingAndThen(
                                Collectors.toCollection(
                                        () -> new TreeSet<>(Comparator.comparing(m -> m.get("new_human_pid") + m.get("old_human_pid")))
                                ), ArrayList::new
                        )
                );
        //需要查询id
        List<String> in = newIds.stream().filter(oldIds::contains).collect(toList());

        return list.stream().peek(m -> {
            String new_human_pid = m.get("new_human_pid");
            if (in.contains(new_human_pid)) {
                String newPid = getNewPid(list, new_human_pid);
                m.put("new_human_pid", newPid);
            }
        }).collect(
                Collectors.collectingAndThen(
                        Collectors.toCollection(
                                () -> new TreeSet<>(Comparator.comparing(m -> m.get("new_human_pid") + m.get("old_human_pid")))
                        ), ArrayList::new
                )
        );
    }

    public static String getNewPid(List<Map<String, String>> list, String newHumanPid) {
        Set<String> tmpIds = new HashSet<>(Collections.singletonList(newHumanPid));
        String tmpId = newHumanPid;
        while (true) {
            String queryId = getQuery(list, tmpId);
            if (tmpIds.contains(queryId) || tmpId.equalsIgnoreCase(queryId)) {
                return queryId;
            }
            tmpId = queryId;
            tmpIds.add(queryId);
        }

    }

    public static String getQuery(List<Map<String, String>> list, String id) {
        Map<String, String> map = list.stream().filter(m -> m.get("old_human_pid").equalsIgnoreCase(id)).findFirst().orElse(null);
        if (map != null && !map.isEmpty()) {
            return map.get("new_human_pid");
        } else {
            return id;
        }
    }

    public static String mergePerson(Session session, List<Map<String, String>> mergeData) {
        String data = null;
        if (!mergeData.isEmpty()) {
            final String merge_cql =
                    "\nWITH  {merge_list} AS batch_list \n" +
                            "UNWIND batch_list AS row\n" +
                            "MATCH (p:" + CompanyEnum.Lable.PERSON.code + " {person_id:row.new_human_pid}),(q:" + CompanyEnum.Lable.PERSON.code + " {person_id:row.old_human_pid})-[r]-(x:" + CompanyEnum.Lable.COMPANY.code + ")\n" +
                            "WITH p as first_node, q as other_node, x, r\n" +
                            "CALL apoc.merge.relationship.eager(first_node, TYPE(r), properties(r),{}, x,{}) YIELD rel\n" +
                            "SET first_node:" + CompanyUtils.getIncrPersonLabelV2("新增", CompanyEnum.SPLIT_HOUR) + "\n" +
                            "SET other_node:" + CompanyUtils.getIncrPersonLabelV2("删除", CompanyEnum.SPLIT_HOUR) + "\n" +
                            "DELETE r\n" +
                            "RETURN first_node.person_id as new_human_pid,first_node.name as new_human_name,other_node.person_id as old_human_pid,other_node.name as old_human_name";
            log.info("merge size: {}, cql2:{}", mergeData.size(), merge_cql);
            data = CompanyUtils.writeNeo4j2(session, merge_cql, new HashMap<String, Object>() {{
                put("merge_list", mergeData);
            }});
        }
        return data;
    }


    public static String getIncrPersonLabel() {
        return CompanyEnum.Lable.新增.code + DateUtil.getDateBefore(-1).replace("-", "");
    }

    public static String getIncrPersonLabel(String label) {
        return label + DateUtil.getDateBefore(-1).replace("-", "");
    }

    public static String getIncrPersonLabelV2(String label) {
        return getIncrPersonLabelV2(label, 10);
    }

    public static String getIncrPersonLabelV2(String label, int split_hour) {
        GregorianCalendar calendar = new GregorianCalendar();
        int now_hour = calendar.get(Calendar.HOUR_OF_DAY);
        int gap = now_hour > split_hour ? 0 : -1;
        return label + DateUtil.getDateBefore(gap).replace("-", "");
    }

    public static Boolean isWindows() {
        return System.getProperty("os.name").contains("Windows");
    }


    public static void main(String[] args) {
        System.out.println(getIncrPersonLabelV2("新增", 10));
        System.out.println(getIncrPersonLabelV2("新增"));
    }
}
